@@ -1,4 +1,5 @@
 /* { dg-do run } */
+/* { dg-skip-if "" { i?86-*-* x86_64-*-* } { "-march=*" } { "-march=amdfam10" } } */
 /* { dg-options "-march=amdfam10 -mno-sse4" } */
 
 extern void abort (void);
@@ -27,7 +28,10 @@ main ()
 #if !defined __SSE4A__
   abort ();
 #endif
-#if defined __SSE5__
+#if defined __AVX__
+  abort ();
+#endif
+#if defined __FMA4__
   abort ();
 #endif
   return 0;
