@@ -86,7 +86,7 @@ void
 fdate (char ** date, gfc_charlen_type * date_len)
 {
   time_t now = time(NULL);
-  *date = get_mem (CTIME_BUFSZ);
+  *date = xmalloc (CTIME_BUFSZ);
   *date_len = gf_ctime (*date, CTIME_BUFSZ, now);
 }
 
@@ -110,7 +110,7 @@ void
 PREFIX(ctime) (char ** date, gfc_charlen_type * date_len, GFC_INTEGER_8 t)
 {
   time_t now = t;
-  *date = get_mem (CTIME_BUFSZ);
+  *date = xmalloc (CTIME_BUFSZ);
   *date_len = gf_ctime (*date, CTIME_BUFSZ, now);
 }
 
