@@ -1,18 +1,19 @@
 #define N 500
 
+int X[2*N], Y[2*N], B[2*N];
+int A[2*N][2*N], C[2*N][2*N];
+
 int foo(void)
 {
   int i, j, k;
-  int X[2*N], Y[2*N], B[2*N];
-  int A[2*N][2*N], C[2*N][2*N];
 
-  for (i = 1; i <= N; i++)
+  for (i = 0; i < N; i++)
     {
       X[i] = Y[i] + 10;
-      for (j = 1; j <= N; j++)
+      for (j = 0; j < N; j++)
 	{
 	  B[j] = A[j][N];
-	  for (k = 1; k <= N; k++)
+	  for (k = 0; k < N; k++)
 	    {
 	      A[j+1][k] = B[j] + C[j][k];
 	    }
