@@ -15,10 +15,10 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
-// { dg-require-fileio "" }
-
 // 27.8.1.3 filebuf member functions
 
+// { dg-require-fileio "" }
+
 #include <locale>
 #include <fstream>
 #include <testsuite_hooks.h>
@@ -46,12 +46,12 @@ protected:
   {
     unshift_called = true;
     to_next = to;
-    return codecvt_base::error;
+    return std::codecvt_base::ok;
   }
 };
 
 // libstdc++/12790
-// basic_filebuf::close() should fail if codecvt::unshift() fails
+// basic_filebuf::close() should call codecvt::unshift()
 void test01()
 {
   using namespace std;
@@ -69,9 +69,8 @@ void test01()
   fb.sputc('a');
 
   VERIFY( !cvt->unshift_called );
-  filebuf* ret = fb.close();
+  fb.close();
   VERIFY( cvt->unshift_called );
-  VERIFY( !ret );
 }
 
 int main()
