@@ -25,32 +25,34 @@
 #include <testsuite_hooks.h>
 
 // libstdc++/12875
-void test02()
+void test01()
 {
   using namespace std;
   bool test __attribute__((unused)) = true;
 
-  const char* name = "tmp_setbuf5";
+  const char* name = "tmp_setbuf4";
   static char buf[1024];
   
-  filebuf out;
-  out.open(name, ios_base::out);
-  streamsize r = out.sputn("Hello,", 6);
-  VERIFY( r == 6 );
-  out.pubsetbuf(buf, 1024);
-  r = out.sputn(" world", 6);
-  VERIFY( r == 6 );
-  VERIFY( out.close() );
+  FILE* out = fopen(name, "w");
+  fputs("Hello, world", out);
+  fclose(out);
   
-  FILE* in = fopen(name, "r");
+  filebuf in;
+  in.open(name, ios_base::in);
   char str[256];
-  VERIFY( fgets(str, 256, in) );
-  VERIFY( !strcmp(str, "Hello, world") );
-  fclose(in);
+  streamsize r = in.sgetn(str, 6);
+  VERIFY( r == 6 );
+  VERIFY( !memcmp(str, "Hello,", 6) );
+  in.pubsetbuf(buf, 1024);
+  r = in.sgetn(str, 6);
+  VERIFY( r == 6 );
+  VERIFY( !memcmp(str, " world", 6) );
+  in.close();
 }
 
+// libstdc++/12875
 int main()
 {
-  test02();
+  test01();
   return 0;
 }
