@@ -1,3 +1,6 @@
+// Derived from libstdc++/12048 by LJR <ljrittle@acm.org> with
+// reminder from Petur Runolfsson <peturr02@ru.is>.
+
 // Copyright (C) 2003-2013 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
@@ -15,8 +18,14 @@
 // with this library; see the file COPYING3.  If not see
 // <http://www.gnu.org/licenses/>.
 
+// DR 49 states that cin.rdbuf()->sbumpc() and fgetc(stdin) should be
+// equivalent and interchangable. Currently however, cin.rdbuf()->sungetc()
+// only returns characters that were read with cin.rdbuf()->sbumpc()
+
 // { dg-require-fileio "" }
 
+// { dg-do run { xfail *-*-* } }
+
 #include <iostream>
 #include <cstdio>
 #include <testsuite_hooks.h>
@@ -26,12 +35,18 @@ test01()
 {
   VERIFY( std::freopen("cin_unget-1.txt", "r", stdin) );
 
-  char buf[2];
-  VERIFY( std::cin.rdbuf()->sgetn(buf, 2) == 2 );
-  int c1 = std::cin.rdbuf()->sungetc();
-  int c2 = std::fgetc(stdin);
-  VERIFY( c1 == std::char_traits<char>::to_int_type(buf[1]) );
-  VERIFY( c2 == c1 );
+  char c1;
+  int c2;
+  char c3;
+  std::cin.get(c1);
+  c2 = std::fgetc(stdin);
+  std::cin.unget();
+  if (std::cin.good())
+    {
+      std::cin.get(c3);
+      VERIFY( std::cin.good() );
+      VERIFY( c3 == std::char_traits<char>::to_char_type(c2) );
+    }
 }
 
 int main(void)
